/*******************************************************************************
 Copyright (c) 2014, Infineon Technologies AG                                 **
 All rights reserved.                                                         **
                                                                              **
 Redistribution and use in source and binary forms, with or without           **
 modification,are permitted provided that the following conditions are met:   **
                                                                              **
 *Redistributions of source code must retain the above copyright notice,      **
 this list of conditions and the following disclaimer.                        **
 *Redistributions in binary form must reproduce the above copyright notice,   **
 this list of conditions and the following disclaimer in the documentation    **
 and/or other materials provided with the distribution.                       **
 *Neither the name of the copyright holders nor the names of its contributors **
 may be used to endorse or promote products derived from this software without**
 specific prior written permission.                                           **
                                                                              **
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  **
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    **
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   **
 ARE  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   **
 LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         **
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF         **
 SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    **
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN      **
 CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       **
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE   **
 POSSIBILITY OF SUCH DAMAGE.                                                  **
                                                                              **
 To improve the quality of the software, users are encouraged to share        **
 modifications, enhancements or bug fixes with Infineon Technologies AG       **
 dave@infineon.com).                                                          **
                                                                              **
********************************************************************************
**                                                                            **
**                                                                            **
** PLATFORM : Infineon XMC4500 Series                                         **
**                                                                            **
** AUTHOR : App Developer                                                     **
**                                                                            **
** version : 1.0.0 (Initial Version)                                     	  **
**                                                                            **
** MODIFICATION DATE : Aug, 25, 2014                                          **
**                                                                            **
*******************************************************************************/

/**
 * @file Main.c
 * @brief Main source file for the SPI001 example 2 demo. This file contains
 * the logic for writing data to the ISOFACE board using SPI communication.
 */

/*******************************************************************************
 ** INCLUDE FILES                                                             **
 ******************************************************************************/

#include <DAVE3.h>			//Declarations from DAVE3 Code Generation (includes SFR declaration)
#include "../../inc/SPI001/Usic.h"

static void Delay(uint32_t delay);
void SPI001_ConfigureSlaveUpdateMode(const SPI001_HandleType* Handle);
bool SPI001_WriteSlave(  const SPI001_HandleType* Handle, const uint16_t* DataPtr, SPI001_CSLineType SlaveSel);
bool SPI001_ReadFromSlave(const SPI001_HandleType* Handle, uint16_t* DataPtr, SPI001_CSLineType SlaveSel);
void SPI001_SetClockPolarity(const SPI001_HandleType* Handle, SPI001_ClkPolType clk_pol);



int main(void)
{
	uint16_t ReadBuf = 0;

	DAVE_Init();			// Initialization of DAVE Apps

	//Configure automatic slave select mode.
	SPI001_ConfigureSlaveUpdateMode(&SPI001_Handle0 );
	while(1)
	{
		//Read status from ISOFACE input at slave select B
		//ISOFACE IN works without clock inversion
		SPI001_SetClockPolarity(&SPI001_Handle0,SPI001_CLK_POL0);
		SPI001_ReadFromSlave(&SPI001_Handle0, &ReadBuf, CE_B);
		Delay(2000);

		//Write the same data to ISOFACE out
		//ISOFACE OUT works with clock inversion
		SPI001_SetClockPolarity(&SPI001_Handle0,SPI001_CLK_POL1);
		SPI001_WriteSlave(&SPI001_Handle0, &ReadBuf, CE_A);
		Delay(2000);
	}
	return 0;
}


/*
 * Function to write data to a slave device.
 */
bool SPI001_WriteSlave(  const SPI001_HandleType* Handle, const uint16_t* DataPtr, SPI001_CSLineType SlaveSel)
{
	  bool Result = (bool)FALSE;
	  uint8_t TbufIndex = SlaveSel;
	  USIC_CH_TypeDef* USICRegs = Handle->USICRegs;

	  if(USIC_IsTxFIFOfull(USICRegs))
	  {
	     Result = (bool)FALSE;
	  }
	  else
	  {
		 //USICRegs->CCR &= (~USIC_CH_CCR_HPCEN_Msk);

		 TbufIndex = (uint8_t)(0x01 << Handle->ChipSelTable[SlaveSel]);
	     USICRegs->IN[TbufIndex] = *DataPtr;
	     Result = (bool)TRUE;
	  }

	  return Result;
}

/*
 * Function sets the clock polarity for SPI communication.
 */
void SPI001_SetClockPolarity(const SPI001_HandleType* Handle, SPI001_ClkPolType clk_pol)
{
	USIC_CH_TypeDef* USICRegs = Handle->USICRegs;
	USICRegs->BRG &= (~(0x01 << USIC_CH_BRG_SCLKCFG_Pos));
	USICRegs->BRG |= clk_pol << USIC_CH_BRG_SCLKCFG_Pos;
}

/*
 * Function to configure automatic slave select.
 */
void SPI001_ConfigureSlaveUpdateMode(const SPI001_HandleType* Handle)
{
	USIC_CH_TypeDef* USICRegs = Handle->USICRegs;
	USICRegs->TCSR &= (~(USIC_CH_TCSR_FLEMD_Msk | USIC_CH_TCSR_HPCMD_Msk | USIC_CH_TCSR_SELMD_Msk | USIC_CH_TCSR_WAMD_Msk | USIC_CH_TCSR_WLEMD_Msk));
	USICRegs->TCSR |= USIC_CH_TCSR_SELMD_Msk;
}

/*
 * Function to clear the read data from a slave device.
 */
bool SPI001_ReadFromSlave(const SPI001_HandleType* Handle, uint16_t* DataPtr, SPI001_CSLineType SlaveSel)
{
	//dummy byte
	uint16_t dummy = 0xff;

	//Clear existing read buffer and flags
	while(SPI001_ReadData(Handle, &dummy) == TRUE)
	{
		SPI001_ClearFlag(Handle, SPI001_FIFO_STD_RECV_BUF_FLAG);
		SPI001_ClearFlag(Handle, SPI001_FIFO_ALTRECV_BUF_FLAG);
	}

	//Clear transmit buffer flags
	SPI001_ClearFlag(Handle, SPI001_FIFO_STD_TRANSMIT_BUF_FLAG);
	SPI001_ClearFlag(Handle, SPI001_FIFO_TRANSMIT_BUF_ERR_FLAG);
	dummy = 0xff;
	SPI001_WriteSlave(Handle, &dummy, SlaveSel);

	//Wait for the receive flag to be set and return the received value
	while(SPI001_ReadData(Handle, DataPtr) == FALSE);

	//Clear the flag and return
	SPI001_ClearFlag(Handle, SPI001_FIFO_STD_RECV_BUF_FLAG);
	SPI001_ClearFlag(Handle, SPI001_FIFO_ALTRECV_BUF_FLAG);

	return TRUE;
}

/*
 * Function to introduce blocking delay
 */
static void Delay(uint32_t delay)
{
	while(delay--);
}
